/***************************************************************************
                          chatserver.cpp  -  description
                             -------------------
    begin                : Mit Apr 18 20:34:30 CEST 2001
    copyright            : (C) 2001 by Emanuel Schmieg
    email                : kerneld@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "chatserver.h"
// Icludes of forward declarated
#include "fileserversocket.h"

ChatServer::ChatServer() : KMainWindow(), files(257)
{
 filenumber = 1;
 KSimpleConfig conf("kchatrc");
 conf.setGroup("Server");
 tmpDir = conf.readEntry("tmpDir", "/tmp/chatserver");
 QString order;
 order = QString("mkdir %1").arg(tmpDir);
 system(order.ascii());
 filesDialog = new FilesDialog(QString(tmpDir), 0);
 fport = 0;
 if (!conf.readBoolEntry("WaitOnTheSamePortAsClient", true))
   port = conf.readNumEntry("waitOnPort", 2000);
 else
   {
    conf.setGroup("Client");
    port = conf.readNumEntry("Port", 2000);
   }

 adminDialog = new SocketAdministration();
 filesDialog = new FilesDialog(tmpDir, 0);
 systemTray = 0;

 startServer();

 resize(600, 540);

 view = new ChatServerView(this);
 setCentralWidget(view);

 FILE *pipe;
 pipe = popen ("echo $LOGNAME", "r");
 char ch;
 while ((ch = fgetc(pipe)) != '\n')
   m_username += ch;
 pclose (pipe);
 char localhost[256];
 gethostname(localhost, 256);
 m_host = localhost;

 Chater* I = new Chater(-1, m_username, m_host);
 Logins.append(I);
 view->newChater(true, I->user());

 KStdAction::configureToolbars (this, SLOT(configToolBar()), actionCollection());

 KAction* pref = KStdAction::preferences (this, SLOT(preferences()), actionCollection());

 saveChatAction = KStdAction::save (this, SLOT(saveChat()), actionCollection());

 hideAction = new KAction (i18n("&Hide"), "exit", Qt::CTRL + Qt::Key_H,
                           this, SLOT (hideThis()),
                           actionCollection(), "file_hide");

 chaterInformationAction = new KAction (i18n("&Information"), "idea", Qt::CTRL + Qt::Key_I,
                                        this, SLOT (information()),
                                        actionCollection(), "chater_information");
 chaterInformationAction->setEnabled(false);

 chaterRemoveAction = new KAction (i18n("&Remove"), "remove", Qt::CTRL + Qt::Key_R,
                                   this, SLOT (remove()),
                                   actionCollection(), "chater_remove");
 chaterRemoveAction->setEnabled(false);

 restartServerAction = new KAction (i18n("R&estart Server"), "reload", Qt::CTRL + Qt::Key_E,
                                    this, SLOT (restart()),
                                    actionCollection(), "preferences_restart");

 chaterBeep = new KAction (i18n("&Beep"), "bell", Qt::CTRL + Qt::Key_B,
                           this, SLOT (beepAll()),
                           actionCollection(), "chater_beep");

 cleanChatAction = new KAction (i18n("&Clean Chat"), "filenew", Qt::CTRL + Qt::Key_C,
                                view, SLOT (cleanChat()), actionCollection(), "clear_chat");

 socketAdministrationAction = new KAction (i18n("Socket &Administration"), "queue",
                                          Qt::CTRL + Qt::Key_A,
                                          this, SLOT (slotSocketAdministration()),
                                          actionCollection(), "socket_administration");

 KAction* startClientAction = new KAction (i18n("Start ChatClient"), "chatclient",
                                          0,
                                          this, SLOT (slotStartClient()),
                                          actionCollection(), "start_chatclient");

 filesAction = new KAction (i18n("Files"), "editcopy",
                            0, this, SLOT (slotFiles()), actionCollection(), "files");

 createGUI();

 QPixmap appIcon((const char **) kchat_pix_data);

 systemTray = new ChatSystemTray(this, "systemtray");
 systemTray->setPixmap(appIcon);

 pref->plug(systemTray->Menu());

 restartServerAction->plug(systemTray->Menu());

 socketAdministrationAction->plug(systemTray->Menu());

 startClientAction->plug(systemTray->Menu());

 systemTray->Menu()->insertTitle(i18n("Port %1, Connections %2").arg(port).arg(0), 0);

 KAction* showAction = new KAction (i18n("Show"), "chatserver", 0,
                           this, SLOT (show()),
                           actionCollection(), "");
 showAction->plug(systemTray->Menu());

 hideAction->plug(systemTray->Menu());

 KAction* shutdownAction = new KAction (i18n("Shutdown Server"), "exit", 0,
                           this, SLOT (slotShutdown()),
                           actionCollection(), "");

 shutdownAction->plug(systemTray->Menu());

 systemTray->show();

 statusBar()->insertItem("", 1, 1);
 statusBar()->insertItem(i18n("Downloads: %1").arg(files.count()), 2, 0, false);
 statusText(i18n("Connections 0"));

 connect(this, SIGNAL (enableSay(bool)),
         view, SLOT (enableSay(bool)));

 connect(view, SIGNAL (newMessage(QString)),
         this, SLOT (ownNewMessage(QString)));
 connect(adminDialog, SIGNAL (closeRequested(int)),
         this, SLOT(slotCloseSocket(int)));
 connect(adminDialog, SIGNAL (testRequested(int)),
         this, SLOT(slotTestSocket(int)));
 connect(adminDialog, SIGNAL (queryRequested(int)),
         this, SLOT(slotSendTestTo(int)));
 connect(filesDialog, SIGNAL(addFile(QString)),
         this, SLOT(slotAddLFile(QString)));
 connect(filesDialog, SIGNAL(removeFile(int)),
         this, SLOT(slotRemoveLFile(int)));
}

ChatServer::~ChatServer()
{
 delete s;
 delete fs;
}
/** catch the newmessage signal of QserverSocket */
void ChatServer::receiveNewMessage(int socketID, QString msg){
 QString rmsg;
 if (msg.at(0) == '@')
   {
    if (msg.at(msg.find(':') + 1) == '@' && msg.contains(':'))
      {
       refused *curr;
       QString loginstring = msg.left(msg.find(':'));
       loginstring.remove(0, 1);
       Chater* current = new Chater(socketID, loginstring);

       for (curr = removed.first();curr != 0;curr = removed.next())
         {
          if (curr->who().ID() == current->ID())
            {
             QString tmp = "@";
             tmp += current->user();
             tmp += '@';
             tmp += current->host();
             tmp += ":~";
             receiveNewMessage(current->ID(), tmp);
             return;
            }
         }
       emit enableSay(true);
       Logins.append(current);
       view->newChater(current->isLinux(), current->user());
       Chater* cur;
       for (cur = Logins.first();cur != 0;cur = Logins.next())
         {
          if (cur->ID() == current->ID())
            break;
          rmsg = '@';
          rmsg += cur->user();
          rmsg += '@';
          rmsg += cur->host();
          rmsg += ":@";
          s->sendToHost(rmsg, current->ID());
         }
       if (!isVisible())
         {
          KApplication::beep();
          show();
         }
      }
    else if (msg.at(msg.find(':') + 1) == '~')
      {
       Chater* current = new Chater(socketID, msg.left(msg.find(':')).remove(0, 1));
       Chater* cur;
       for(cur = Logins.first();cur != 0; cur = Logins.next())
            {
             if(cur->ID() == current->ID())
               Logins.remove(cur);
            }
       view->removeChater(current->user());
      }
    else if (msg.at(msg.find(':') + 1) == 'b')
      KApplication::beep();
    else if ((msg.at(msg.find(':') + 1) == 'a') && querySending)
      {
       querySending = false;
       KMessageBox::information(adminDialog,
          i18n("The Host answered after %1 msec").arg(queryTime.msecsTo(QTime::currentTime())),
          i18n("Query answer"));
       return;
      }
    else if (msg == QString("@fileport"))
      {
       s->sendToHost(QString("@fileport %1").arg(fport), socketID);
       return;
      }
    else if (msg == QString("@downloads"))
      {
       s->sendToHost(QString("@downloads %1").arg(files.count()), socketID);
       return;
      }
    else if (msg == QString("@filenames"))
      {
       QIntDictIterator <QFile> it(files);
       for (QFile* file = it.toFirst(); file;file = ++it)
         s->sendToHost(QString("@filename %1#%2").arg(it.currentKey())
           .arg(file->name().right(file->name().length() - file->name().findRev('/') - 1)), socketID);
       return;
      }
   }
 else
   {
    view->displayNewMessage(msg);
   }
 s->sendAllHosts(msg);
}
void ChatServer::connectionnumberchanged(int number){
 if (number == 0)
   {
    emit enableSay(false);
    restartServerAction->setEnabled(true);
    chaterInformationAction->setEnabled(false);
    chaterRemoveAction->setEnabled(false);
    chaterBeep->setEnabled(false);
   }
 else
   {
    restartServerAction->setEnabled(false);
    chaterInformationAction->setEnabled(true);
    chaterRemoveAction->setEnabled(true);
    chaterBeep->setEnabled(true);
   }
 statusText(i18n("Connections: %1").arg(number));
 systemTray->Menu()->changeTitle(0, i18n("Port %1, Connections %2").arg(port).arg(number));
}
void ChatServer::ownNewMessage(QString msg){
 if (msg == "")
   {
    view->displayNewMessage(i18n("Please don't send this message again"));
    return;
   }
 QString tmp = m_username;
 tmp += '@';
 tmp += m_host;
 tmp += ": ";
 tmp += msg;
 receiveNewMessage (-1, tmp);
}

void ChatServer::hideThis(){
 hide();
 view->cleanChat();
}

bool ChatServer::queryClose(){
 hideThis();
 return false;
}

void ChatServer::startServer(){
 s = new ChatServerSocket(port, 0, this, 0);
 fs = new FileServerSocket(0, 0, this, 0);
 if (!s->ok())
   {
    delete s;
    s = 0;
    delete fs;
    fs = 0;
    ErrorConnecting* dialog = new ErrorConnecting(port, false, this);
    if (dialog->exec() == QDialog::Accepted)
      {
       port = dialog->getPort();
       startServer();
       return;
      }
    else
      exit(-1);
   }
 if (!fs->ok())
   {
    delete s;
    s = 0;
    delete fs;
    fs = 0;
    ErrorConnecting* dialog = new ErrorConnecting(fport, true, this);
    if (dialog->exec() == QDialog::Accepted)
      {
       fport = dialog->getPort();
       startServer();
       return;
      }
    else
      exit(-1);
   }
 fport = fs->port();
 if (systemTray != 0)
   systemTray->Menu()->changeTitle(0, i18n("Port %1, Connections %2").arg(port).arg(0));
 connect(s, SIGNAL (newmessage(int, QString)),
         this, SLOT (receiveNewMessage(int, QString)));

 connect(s, SIGNAL (connectionCounterChanged(int)),
         this, SLOT (connectionnumberchanged(int)));

 connect(s, SIGNAL (socketsChanged(QList <ChatSocket>)),
         adminDialog, SLOT(socketsChanged(QList <ChatSocket>)));
}
void ChatServer::configToolBar(){
 KEditToolbar dialog (actionCollection());
 if (dialog.exec())
   createGUI();
}
/** removes the current chater */
void ChatServer::remove(){
 banishChater *dialog = new banishChater(this);
 if (dialog->exec() == QDialog::Accepted)
   {
    Chater* current;
    QString tmp = "@";
    for(current = Logins.first();current != 0; current = Logins.next())
      {
       if (current->user() == view->getCurrentText())
         {
          tmp += current->user();
          tmp += '@';
          tmp += current->host();
          tmp += ":~";
          receiveNewMessage(current->ID(), tmp);
          int id;
          id = startTimer(dialog->timeout() * 60 * 1000);
          refused *cur = new refused(current->user(), current->host(), id, current->ID());
          removed.append(cur);
          break;
         }
      }
   }
}
void ChatServer::information(){
 Chater* current;
 QString text = i18n("User: %1\nHost: %2");
 for(current = Logins.first();current != 0; current = Logins.next())
   {
    if (current->user() == view->getCurrentText())
      {
       KMessageBox::information (this, text.arg(current->user()).arg(current->host()));
       break;
      }
   }
}
/** calls the preferences */
void ChatServer::preferences(){
 kcmchat * dialog = new kcmchat(this, "Preferences", true);
 dialog->exec();
}
void ChatServer::restart(){
 statusText(i18n("Restarting Server"));
 delete s;
 s = 0;
 KSimpleConfig conf("kchatrc");
 conf.setGroup("Server");
 if (!conf.readBoolEntry("WaitOnTheSamePortAsClient", true))
   port = conf.readNumEntry("waitOnPort", 2000);
 else
   {
    conf.setGroup("Client");
    port = conf.readNumEntry("Port", 2000);
   }
 startServer();
 statusText(i18n("Connections: 0"));
}
void ChatServer::beepAll(){
 QString tmp = "";
 tmp += '@';
 tmp += m_username;
 tmp += '@';
 tmp += m_host;
 tmp += ":b";
 receiveNewMessage (-1, tmp);
}
void ChatServer::timerEvent(QTimerEvent *ev){
 killTimer(ev->timerId());
 refused *cur;
 int i = 0;
 for (cur = removed.first();cur != 0;cur = removed.next(), i++)
   {
    if (ev->timerId() == cur->ID())
      {
       removed.remove(i);
       QString text;
       text = i18n("Banish over for %1");
       KMessageBox::information (this, text.arg(cur->who().user()));
       break;
      }
   }
}
void ChatServer::saveChat(){
 KURL url;
 url = KFileDialog::getSaveURL ();
 if (url.isEmpty())
   return;
 if (url.isMalformed())
   {
    QString text = i18n("The URL %1 is not correct!");
    KMessageBox::sorry (this, text.arg(url.prettyURL()));
    return;
   }
 if (url.isLocalFile())
   {
    QFile file(url.path());
    file.open(IO_WriteOnly);
    saveToLocalFile (&file);
   }
 else
   {
    KTempFile tempfile;
    saveToLocalFile (tempfile.file());
    if (!KIO::NetAccess::upload (tempfile.name(), url))
      {
       QString text = i18n("Error uploading %1");
       KMessageBox::sorry (this, text.arg(url.prettyURL()));
       tempfile.unlink();
       return;
      }
    tempfile.unlink();
   }
}
void ChatServer::saveToLocalFile(QFile *file){
 QString text;
 text = i18n("Chat-protocol of ");
 text += KGlobal::locale()->formatDateTime (QDateTime::currentDateTime(), false, true);
 text += '\n';
 text += view->ChatText();
 QTextStream stream (file);
 stream.setEncoding(QTextStream::Latin1);
 stream << text;
 file->close();
}
/** open the socket Administration dialog */
void ChatServer::slotSocketAdministration(){
 adminDialog->show();
}
/** test the socket number pos */
void ChatServer::slotTestSocket(int id){
 if (s->getSocket(id) != 0)
   {
    switch(s->getSocket(id)->state())
      {
       case QSocket::Idle       : KApplication::beep();
                                  KMessageBox::error(adminDialog, i18n("The Socket is not connected!!"),
                                                     i18n("test result"));
                                  break;
       case QSocket::HostLookup : KApplication::beep();
                                  KMessageBox::error(adminDialog, i18n("The Socket is looking up a host!!"),
                                                     i18n("test result"));
                                  break;
       case QSocket::Connecting : KApplication::beep();
                                  KMessageBox::error(adminDialog, i18n("The Socket is connecting!!"),
                                                     i18n("test result"));
                                  break;
       case QSocket::Connection : KMessageBox::information(adminDialog, i18n("The Socket is connected normally."),
                                                     i18n("test result"));
                                  break;
       case QSocket::Listening  : KMessageBox::information(adminDialog, i18n("The Socket is read."),
                                                     i18n("test result"));
                                  break;
       case QSocket::Closing    : KMessageBox::information(adminDialog, i18n("The Socket closes the connection."),
                                                     i18n("test result"));
                                  break;
      }
   }
}
void ChatServer::slotSendTestTo(int id){
 querySending = false;
 QString msg = "@";
 msg += m_username;
 msg += '@';
 msg += m_host;
 msg += ":query";
 s->sendToHost(msg, id);
 queryTime = QTime::currentTime();
 querySending = true;
}
void ChatServer::slotCloseSocket(int id){
 s->closeSocket(id);
}
/** starts the client */
void ChatServer::slotStartClient(){
 KProcess* proc = new KProcess();
 (*proc) << "chatclient";
 proc->start (KProcess::DontCare);
}
/** shuts the server down */
void ChatServer::slotShutdown(){
 if (KMessageBox::questionYesNo(this, i18n("Should I really shutdown the server?\n"
                                           "You won't be able to get any connection!!\n"
                                           "Use File->hide to hide the server."),
                                      i18n("really shutdown?")) == KMessageBox::Yes)
   kapp->quit();
}
/** displays text in the statusbar */
void ChatServer::statusText(QString text){
 statusBar()->changeItem(text, 1);
}
/** No descriptions */
QFile* ChatServer::addFile(QString filename){
 KSimpleConfig conf("kchatrc");
 conf.setGroup("Server");
 if (conf.readBoolEntry("askForUpload", true))
   if (KMessageBox::questionYesNo(this, i18n("Would you like do load %1 up?").arg(filename),
      i18n("Upload?")) == KMessageBox::No)
      return 0;
 QString fname;
 fname = tmpDir;
 fname += '/';
 fname += filename;
 return new QFile(fname);
}
/** No descriptions */
QFile* ChatServer::getFile(int number){
 return files.find(number);
}

void ChatServer::uploadReady(QFile* file){
 files.insert(filenumber, file);
 statusBar()->changeItem(i18n("Downloads: %1").arg(files.count()), 2);
 s->sendAllHosts(QString("@downloads %1").arg(files.count()));
 QIntDictIterator <QFile> it(files);
 filesDialog->clear();
 for (QFile* file = it.toFirst(); file;file = ++it)
   filesDialog->appendFile(it.currentKey(),
     file->name().right(file->name().length() - file->name().findRev('/') - 1));
 for (QFile* file = it.toFirst(); file;file = ++it)
   s->sendAllHosts(QString("@filename %1#%2").arg(it.currentKey())
           .arg(file->name().right(file->name().length() - file->name().findRev('/') - 1)));
 filenumber++;
}
/** No descriptions */
void ChatServer::slotFiles(){
 filesDialog->show();
}
void ChatServer::slotAddLFile(QString filename){
 uploadReady(addFile(filename));
}
void ChatServer::slotRemoveLFile(int id){
 files.remove(id);
 filesDialog->clear();
 QIntDictIterator <QFile> it(files);
 for (QFile* file = it.toFirst(); file;file = ++it)
   filesDialog->appendFile(it.currentKey(),
     file->name().right(file->name().length() - file->name().findRev('/') - 1));
 s->sendAllHosts("@clear");
 statusBar()->changeItem(i18n("Downloads: %1").arg(files.count()), 2);
}
