/***************************************************************************
                          chatserver.h  -  description
                             -------------------
    begin                : Mit Apr 18 20:34:30 CEST 2001
    copyright            : (C) 2001 by Emanuel Schmieg
    email                : kerneld@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CHATSERVER_H
#define CHATSERVER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapp.h>
#include <qwidget.h>
#include <qmessagebox.h>
#include <kmainwindow.h>
#include <ksimpleconfig.h>
#include <klocale.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kedittoolbar.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kio/netaccess.h>
#include <ktempfile.h>
#include <kprocess.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qdatetime.h>
#include <qintdict.h>
#include <qfile.h>

#include "chatserverview.h"
#include "chatserversocket.h"
#include "chater.h"
#include "kcmchat.h"
#include "banishchater.h"
#include "refused.h"
#include "socketadministration.h"
#include "chatsystemtray.h"
#include "errorconnecting.h"
#include "kchat.h"
#include "filesdialog.h"

#include <stdlib.h>
#include <iostream.h>
#include <stdio.h>
#include <unistd.h>

/// Forward deklarated
class FileServerSocket;

/** ChatServer is the base class of the project */
class ChatServer : public KMainWindow
{
  Q_OBJECT 
  public:
    /** construtor */
    ChatServer();
    /** destructor */
    ~ChatServer();
    /** returns the number of filename or 0 if not added */
    QFile* addFile(QString filename);
    /** No descriptions */
    QFile* getFile(int number);
    void uploadReady(QFile* file);
  signals: // Signals
    /** signals that there is a new message */
    void newMessage(QString msg);
    /** No descriptions */
    void enableSay(bool enable);
  protected:
    bool queryClose();
  /** No descriptions */
  void timerEvent(QTimerEvent *ev);
  private:
    void startServer();
    void saveToLocalFile(QFile *file);
  /** displays text in the statusbar */
  void statusText(QString text);

    ChatServerView* view;
    SocketAdministration* adminDialog;
    FilesDialog* filesDialog;

    ChatServerSocket* s;
    int port;
    FileServerSocket* fs;
    int fport;

    QString m_username;
    QString m_host;
    QList <Chater> Logins;
    QList <refused> removed;

    KAction* hideAction;
    KAction* chaterRemoveAction;
    KAction* chaterInformationAction;
    KAction* restartServerAction;
    KAction* chaterBeep;
    KAction* saveChatAction;
    KAction* cleanChatAction;
    KAction* socketAdministrationAction;
    KAction* filesAction;
    ChatSystemTray* systemTray;

    QTime queryTime;
    bool querySending;

    QIntDict <QFile> files;
    int filenumber;
    QString tmpDir;

  private slots: // Private slots
    /** catch the newmessage signal of QserverSocket */
    void receiveNewMessage(int socketID, QString msg);
    void connectionnumberchanged(int number);
    void ownNewMessage(QString msg);
    void hideThis();
    void configToolBar();
    void information();
    /** removes the current chater */
    void remove();
    /** calls the preferences */
    void preferences();
    void beepAll();
    void restart();
    /** open the socket Administration dialog */
    void slotSocketAdministration();
    void saveChat();
    void slotSendTestTo(int id);
    /** test the socket number id */
    void slotTestSocket(int id);
    void slotCloseSocket(int id);
  /** starts the client */
  void slotStartClient();
  /** shuts the server down */
  void slotShutdown();
  /** No descriptions */
  void slotFiles();
  /** No descriptions */
  void slotRemoveLFile(int id);
  /** No descriptions */
  void slotAddLFile(QString filename);
};

#endif
