/***************************************************************************
                          filesocket.cpp  -  description
                             -------------------
    begin                : Wed Jan 2 2002
    copyright            : (C) 2002 by Emanuel Schmieg
    email                : kerneld@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "filesocket.h"
#include <kapp.h>

FileSocket::FileSocket(ChatServer* server, QObject *parent, const char *name )
 : QSocket(parent,name) {
 bytes = 0;
 transfer = false;
 mode = Undefined;
 fileopen = false;
 program = server;
 file = 0;
 connect(this, SIGNAL (readyRead()), this, SLOT(receiveData()));
}
FileSocket::~FileSocket(){
}
void FileSocket::deleteMyself(){
	// Let the socket object delete itself
	// This is something like a hack, since deleting an object while
	// running its own method is not very nice. But since after the
	// delete we are not using any member variables or virtual methods
	// of the object, this is safe.
	// Of course this slot may only be called for objects that were
	// created with "new".

 delete this;
}
void FileSocket::receiveData ()
{
 // This slot does the read server work!
 while ((bytes == 0) && this->canReadLine())
   {
    QString command;
    command = this->readLine();
    if (command.at(0) != '@')
      return;
    if ((command.left(command.find(' ')) == QString("@bytes")) && (mode == Put) && fileopen)
      {
       bytes = command.right(command.length() - (command.find(' ') + 1)).toInt();
       packets = (bytes / 1024) + 1;
       fullsteps = packets;
       pDialog = new QProgressDialog(
         i18n("Uploading %1").arg(file->name().right(file->name().length()
         - (file->name().findRev('/') + 1))), i18n("Cancel"), fullsteps, 0, i18n("Upload"), false);
       startTimer(0);
      }
    if (command.left(command.find(' ')) == QString("@put"))
      {
       mode = Put;
       filename = command.right(command.length() - (command.find(' ') + 1));
       filename.truncate (filename.length() - 1);
       file = program->addFile(filename);
       if(file)
         {
          if (file->open(IO_WriteOnly))
            {
             QString answer;
             answer = QString("@allowed\n");
             this->writeBlock(answer.ascii(), answer.length());
             fileopen = true;
            }
          else
            {
             QString answer;
             answer = QString("@error\n");
             this->writeBlock(answer.ascii(), answer.length());
            }
         }
       else
         {
          QString answer;
          answer = QString("@forbidden\n");
          this->writeBlock(answer.ascii(), answer.length());
         }
      }
    if (command.left(command.find(' ')) == QString("@get"))
      {
       mode = Get;
       filenumber = command.right(command.length() - (command.find(' ') + 1)).toInt();
       file = program->getFile(filenumber);
       if (file)
         {
          if (!file->open(IO_ReadOnly))
            return;
          bytes = file->size();
          QString answer;
          answer = QString("@bytes %1\n").arg(bytes);
          this->writeBlock(answer.ascii(), answer.length());
          packets = (bytes / 1024) + 1;
          fullsteps = packets;
          pDialog = new QProgressDialog(
            i18n("Downloading %1").arg(file->name().right(file->name().length()
            - (file->name().findRev('/') + 1))), i18n("Cancel"), fullsteps, 0, i18n("Download"), false);
          startTimer(0);
         }
       else
         {
          QString answer;
          answer = QString("@forbidden\n");
          this->writeBlock(answer.ascii(), answer.length());
         }
      }
   }
}
void FileSocket::timerEvent(QTimerEvent*)
{
 while ((bytes - (packets-1)*1024) &&
  (((bytesAvailable() > 0) && (mode == Put))
  || ((!file->atEnd()) && (mode == Get))))
   {
    if (mode == Get)
      this->putch(file->getch());
    else if (mode == Put)
      file->putch(this->getch());
    bytes--;
   }
 if (packets)
   {
    pDialog->setProgress(fullsteps - packets);
    if (pDialog->wasCancelled())
      QTimer::singleShot(0, this, SLOT(deleteMyself()));
   }
 else
   {
    delete pDialog;
    killTimers();
    file->close();
    if (mode == Put)
      {
       program->uploadReady(file);
       this->close();
      }
   }
 if ((bytes - (packets-1)*1024) <= 0)
   packets--;
}
