/***************************************************************************
                          filesocket.h  -  description
                             -------------------
    begin                : Wed Jan 2 2002
    copyright            : (C) 2002 by Emanuel Schmieg
    email                : kerneld@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FILESOCKET_H
#define FILESOCKET_H

#include <qwidget.h>
#include <qsocket.h>
#include <qprogressdialog.h>
#include "chatserver.h"

/**the socket which handles files
  *@author Emanuel Schmieg
  */

class FileSocket : public QSocket  {
   Q_OBJECT

public:
	FileSocket(ChatServer* server, QObject *parent=0, const char *name=0);
	~FileSocket();

public slots:	
	/**
	  This little helping slot deletes the object itself. So it may
		only be called for objects created with new!
		It is used because the deletion is triggered by the signal
		connectionClosed. It could lead to a segfault if we deleted the
		object directly, since control comes back to that signal. So
		instead we start a zero-timer, that activates this slot. The
		zero-timer has its timeout not before we returned to the main
		event loop, and then it is safe to delete the object.
		See "FileServerSocket::oneConnectionClosed()" for details of the
		implementation.
	*/
	void deleteMyself ();	

protected:
  void timerEvent(QTimerEvent*);

private slots:
	/**
	  This private slot is called from the signal "readyRead", that
	  shows that new data is waiting for us.
	*/
	void receiveData ();

private:
  enum Mode {Undefined, Get, Put};

  ChatServer* program;
  QProgressDialog* pDialog;
  int bytes;
  int packets;
  int fullsteps;
  bool transfer;
  Mode mode;
  int filenumber;
  QString filename;
  QFile* file;
  bool fileopen;
};

#endif
